# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum


class ItemTypeEnum(StrEnum):
    APP = "APP"
    BUNDLE = "BUNDLE"
    CODE = "CODE"
    COINS = "COINS"
    EXTENSION = "EXTENSION"
    INGAMEITEM = "INGAMEITEM"
    LOOTBOX = "LOOTBOX"
    MEDIA = "MEDIA"
    OPTIONBOX = "OPTIONBOX"
    SEASON = "SEASON"
    SUBSCRIPTION = "SUBSCRIPTION"


class FulfillmentItem(Model):
    """Fulfillment item (FulfillmentItem)

    Properties:
        quantity: (quantity) REQUIRED int

        extra_subscription_days: (extraSubscriptionDays) OPTIONAL int

        item_id: (itemId) OPTIONAL str

        item_name: (itemName) OPTIONAL str

        item_sku: (itemSku) OPTIONAL str

        item_type: (itemType) OPTIONAL Union[str, ItemTypeEnum]

        store_id: (storeId) OPTIONAL str
    """

    # region fields

    quantity: int  # REQUIRED
    extra_subscription_days: int  # OPTIONAL
    item_id: str  # OPTIONAL
    item_name: str  # OPTIONAL
    item_sku: str  # OPTIONAL
    item_type: Union[str, ItemTypeEnum]  # OPTIONAL
    store_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_quantity(self, value: int) -> FulfillmentItem:
        self.quantity = value
        return self

    def with_extra_subscription_days(self, value: int) -> FulfillmentItem:
        self.extra_subscription_days = value
        return self

    def with_item_id(self, value: str) -> FulfillmentItem:
        self.item_id = value
        return self

    def with_item_name(self, value: str) -> FulfillmentItem:
        self.item_name = value
        return self

    def with_item_sku(self, value: str) -> FulfillmentItem:
        self.item_sku = value
        return self

    def with_item_type(self, value: Union[str, ItemTypeEnum]) -> FulfillmentItem:
        self.item_type = value
        return self

    def with_store_id(self, value: str) -> FulfillmentItem:
        self.store_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "quantity"):
            result["quantity"] = int(self.quantity)
        elif include_empty:
            result["quantity"] = 0
        if hasattr(self, "extra_subscription_days"):
            result["extraSubscriptionDays"] = int(self.extra_subscription_days)
        elif include_empty:
            result["extraSubscriptionDays"] = 0
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "item_name"):
            result["itemName"] = str(self.item_name)
        elif include_empty:
            result["itemName"] = ""
        if hasattr(self, "item_sku"):
            result["itemSku"] = str(self.item_sku)
        elif include_empty:
            result["itemSku"] = ""
        if hasattr(self, "item_type"):
            result["itemType"] = str(self.item_type)
        elif include_empty:
            result["itemType"] = Union[str, ItemTypeEnum]()
        if hasattr(self, "store_id"):
            result["storeId"] = str(self.store_id)
        elif include_empty:
            result["storeId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        quantity: int,
        extra_subscription_days: Optional[int] = None,
        item_id: Optional[str] = None,
        item_name: Optional[str] = None,
        item_sku: Optional[str] = None,
        item_type: Optional[Union[str, ItemTypeEnum]] = None,
        store_id: Optional[str] = None,
        **kwargs,
    ) -> FulfillmentItem:
        instance = cls()
        instance.quantity = quantity
        if extra_subscription_days is not None:
            instance.extra_subscription_days = extra_subscription_days
        if item_id is not None:
            instance.item_id = item_id
        if item_name is not None:
            instance.item_name = item_name
        if item_sku is not None:
            instance.item_sku = item_sku
        if item_type is not None:
            instance.item_type = item_type
        if store_id is not None:
            instance.store_id = store_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> FulfillmentItem:
        instance = cls()
        if not dict_:
            return instance
        if "quantity" in dict_ and dict_["quantity"] is not None:
            instance.quantity = int(dict_["quantity"])
        elif include_empty:
            instance.quantity = 0
        if (
            "extraSubscriptionDays" in dict_
            and dict_["extraSubscriptionDays"] is not None
        ):
            instance.extra_subscription_days = int(dict_["extraSubscriptionDays"])
        elif include_empty:
            instance.extra_subscription_days = 0
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "itemName" in dict_ and dict_["itemName"] is not None:
            instance.item_name = str(dict_["itemName"])
        elif include_empty:
            instance.item_name = ""
        if "itemSku" in dict_ and dict_["itemSku"] is not None:
            instance.item_sku = str(dict_["itemSku"])
        elif include_empty:
            instance.item_sku = ""
        if "itemType" in dict_ and dict_["itemType"] is not None:
            instance.item_type = str(dict_["itemType"])
        elif include_empty:
            instance.item_type = Union[str, ItemTypeEnum]()
        if "storeId" in dict_ and dict_["storeId"] is not None:
            instance.store_id = str(dict_["storeId"])
        elif include_empty:
            instance.store_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, FulfillmentItem]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[FulfillmentItem]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[FulfillmentItem, List[FulfillmentItem], Dict[Any, FulfillmentItem]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "quantity": "quantity",
            "extraSubscriptionDays": "extra_subscription_days",
            "itemId": "item_id",
            "itemName": "item_name",
            "itemSku": "item_sku",
            "itemType": "item_type",
            "storeId": "store_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "quantity": True,
            "extraSubscriptionDays": False,
            "itemId": False,
            "itemName": False,
            "itemSku": False,
            "itemType": False,
            "storeId": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "itemType": [
                "APP",
                "BUNDLE",
                "CODE",
                "COINS",
                "EXTENSION",
                "INGAMEITEM",
                "LOOTBOX",
                "MEDIA",
                "OPTIONBOX",
                "SEASON",
                "SUBSCRIPTION",
            ],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class SimpleEntitlement(Model):
    """Simple entitlement (SimpleEntitlement)

    Properties:
        entitlement_id: (entitlementId) OPTIONAL str

        granted_code: (grantedCode) OPTIONAL str

        item_id: (itemId) OPTIONAL str

        item_namespace: (itemNamespace) OPTIONAL str

        name: (name) OPTIONAL str

        namespace: (namespace) OPTIONAL str

        origin: (origin) OPTIONAL str

        sku: (sku) OPTIONAL str

        use_count: (useCount) OPTIONAL int
    """

    # region fields

    entitlement_id: str  # OPTIONAL
    granted_code: str  # OPTIONAL
    item_id: str  # OPTIONAL
    item_namespace: str  # OPTIONAL
    name: str  # OPTIONAL
    namespace: str  # OPTIONAL
    origin: str  # OPTIONAL
    sku: str  # OPTIONAL
    use_count: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_entitlement_id(self, value: str) -> SimpleEntitlement:
        self.entitlement_id = value
        return self

    def with_granted_code(self, value: str) -> SimpleEntitlement:
        self.granted_code = value
        return self

    def with_item_id(self, value: str) -> SimpleEntitlement:
        self.item_id = value
        return self

    def with_item_namespace(self, value: str) -> SimpleEntitlement:
        self.item_namespace = value
        return self

    def with_name(self, value: str) -> SimpleEntitlement:
        self.name = value
        return self

    def with_namespace(self, value: str) -> SimpleEntitlement:
        self.namespace = value
        return self

    def with_origin(self, value: str) -> SimpleEntitlement:
        self.origin = value
        return self

    def with_sku(self, value: str) -> SimpleEntitlement:
        self.sku = value
        return self

    def with_use_count(self, value: int) -> SimpleEntitlement:
        self.use_count = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "entitlement_id"):
            result["entitlementId"] = str(self.entitlement_id)
        elif include_empty:
            result["entitlementId"] = ""
        if hasattr(self, "granted_code"):
            result["grantedCode"] = str(self.granted_code)
        elif include_empty:
            result["grantedCode"] = ""
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "item_namespace"):
            result["itemNamespace"] = str(self.item_namespace)
        elif include_empty:
            result["itemNamespace"] = ""
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "origin"):
            result["origin"] = str(self.origin)
        elif include_empty:
            result["origin"] = ""
        if hasattr(self, "sku"):
            result["sku"] = str(self.sku)
        elif include_empty:
            result["sku"] = ""
        if hasattr(self, "use_count"):
            result["useCount"] = int(self.use_count)
        elif include_empty:
            result["useCount"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        entitlement_id: Optional[str] = None,
        granted_code: Optional[str] = None,
        item_id: Optional[str] = None,
        item_namespace: Optional[str] = None,
        name: Optional[str] = None,
        namespace: Optional[str] = None,
        origin: Optional[str] = None,
        sku: Optional[str] = None,
        use_count: Optional[int] = None,
        **kwargs,
    ) -> SimpleEntitlement:
        instance = cls()
        if entitlement_id is not None:
            instance.entitlement_id = entitlement_id
        if granted_code is not None:
            instance.granted_code = granted_code
        if item_id is not None:
            instance.item_id = item_id
        if item_namespace is not None:
            instance.item_namespace = item_namespace
        if name is not None:
            instance.name = name
        if namespace is not None:
            instance.namespace = namespace
        if origin is not None:
            instance.origin = origin
        if sku is not None:
            instance.sku = sku
        if use_count is not None:
            instance.use_count = use_count
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> SimpleEntitlement:
        instance = cls()
        if not dict_:
            return instance
        if "entitlementId" in dict_ and dict_["entitlementId"] is not None:
            instance.entitlement_id = str(dict_["entitlementId"])
        elif include_empty:
            instance.entitlement_id = ""
        if "grantedCode" in dict_ and dict_["grantedCode"] is not None:
            instance.granted_code = str(dict_["grantedCode"])
        elif include_empty:
            instance.granted_code = ""
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "itemNamespace" in dict_ and dict_["itemNamespace"] is not None:
            instance.item_namespace = str(dict_["itemNamespace"])
        elif include_empty:
            instance.item_namespace = ""
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "origin" in dict_ and dict_["origin"] is not None:
            instance.origin = str(dict_["origin"])
        elif include_empty:
            instance.origin = ""
        if "sku" in dict_ and dict_["sku"] is not None:
            instance.sku = str(dict_["sku"])
        elif include_empty:
            instance.sku = ""
        if "useCount" in dict_ and dict_["useCount"] is not None:
            instance.use_count = int(dict_["useCount"])
        elif include_empty:
            instance.use_count = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, SimpleEntitlement]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[SimpleEntitlement]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        SimpleEntitlement, List[SimpleEntitlement], Dict[Any, SimpleEntitlement]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "entitlementId": "entitlement_id",
            "grantedCode": "granted_code",
            "itemId": "item_id",
            "itemNamespace": "item_namespace",
            "name": "name",
            "namespace": "namespace",
            "origin": "origin",
            "sku": "sku",
            "useCount": "use_count",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "entitlementId": False,
            "grantedCode": False,
            "itemId": False,
            "itemNamespace": False,
            "name": False,
            "namespace": False,
            "origin": False,
            "sku": False,
            "useCount": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum

from ..models.credit_payload import CreditPayload
from ..models.debit_payload import DebitPayload
from ..models.ful_fill_item_payload import FulFillItemPayload
from ..models.revoke_entitlement_payload import RevokeEntitlementPayload


class TypeEnum(StrEnum):
    CREDIT_WALLET = "CREDIT_WALLET"
    DEBIT_WALLET = "DEBIT_WALLET"
    FULFILL_ITEM = "FULFILL_ITEM"
    REVOKE_ENTITLEMENT = "REVOKE_ENTITLEMENT"


class OperationRequest(Model):
    """Represents a single operation request. (OperationRequest)

    Properties:
        credit_payload: (creditPayload) OPTIONAL CreditPayload

        debit_payload: (debitPayload) OPTIONAL DebitPayload

        ful_fill_item_payload: (fulFillItemPayload) OPTIONAL FulFillItemPayload

        revoke_entitlement_payload: (revokeEntitlementPayload) OPTIONAL RevokeEntitlementPayload

        type_: (type) OPTIONAL Union[str, TypeEnum]
    """

    # region fields

    credit_payload: CreditPayload  # OPTIONAL
    debit_payload: DebitPayload  # OPTIONAL
    ful_fill_item_payload: FulFillItemPayload  # OPTIONAL
    revoke_entitlement_payload: RevokeEntitlementPayload  # OPTIONAL
    type_: Union[str, TypeEnum]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_credit_payload(self, value: CreditPayload) -> OperationRequest:
        self.credit_payload = value
        return self

    def with_debit_payload(self, value: DebitPayload) -> OperationRequest:
        self.debit_payload = value
        return self

    def with_ful_fill_item_payload(self, value: FulFillItemPayload) -> OperationRequest:
        self.ful_fill_item_payload = value
        return self

    def with_revoke_entitlement_payload(
        self, value: RevokeEntitlementPayload
    ) -> OperationRequest:
        self.revoke_entitlement_payload = value
        return self

    def with_type(self, value: Union[str, TypeEnum]) -> OperationRequest:
        self.type_ = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "credit_payload"):
            result["creditPayload"] = self.credit_payload.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["creditPayload"] = CreditPayload()
        if hasattr(self, "debit_payload"):
            result["debitPayload"] = self.debit_payload.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["debitPayload"] = DebitPayload()
        if hasattr(self, "ful_fill_item_payload"):
            result["fulFillItemPayload"] = self.ful_fill_item_payload.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["fulFillItemPayload"] = FulFillItemPayload()
        if hasattr(self, "revoke_entitlement_payload"):
            result[
                "revokeEntitlementPayload"
            ] = self.revoke_entitlement_payload.to_dict(include_empty=include_empty)
        elif include_empty:
            result["revokeEntitlementPayload"] = RevokeEntitlementPayload()
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = Union[str, TypeEnum]()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        credit_payload: Optional[CreditPayload] = None,
        debit_payload: Optional[DebitPayload] = None,
        ful_fill_item_payload: Optional[FulFillItemPayload] = None,
        revoke_entitlement_payload: Optional[RevokeEntitlementPayload] = None,
        type_: Optional[Union[str, TypeEnum]] = None,
        **kwargs,
    ) -> OperationRequest:
        instance = cls()
        if credit_payload is not None:
            instance.credit_payload = credit_payload
        if debit_payload is not None:
            instance.debit_payload = debit_payload
        if ful_fill_item_payload is not None:
            instance.ful_fill_item_payload = ful_fill_item_payload
        if revoke_entitlement_payload is not None:
            instance.revoke_entitlement_payload = revoke_entitlement_payload
        if type_ is not None:
            instance.type_ = type_
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> OperationRequest:
        instance = cls()
        if not dict_:
            return instance
        if "creditPayload" in dict_ and dict_["creditPayload"] is not None:
            instance.credit_payload = CreditPayload.create_from_dict(
                dict_["creditPayload"], include_empty=include_empty
            )
        elif include_empty:
            instance.credit_payload = CreditPayload()
        if "debitPayload" in dict_ and dict_["debitPayload"] is not None:
            instance.debit_payload = DebitPayload.create_from_dict(
                dict_["debitPayload"], include_empty=include_empty
            )
        elif include_empty:
            instance.debit_payload = DebitPayload()
        if "fulFillItemPayload" in dict_ and dict_["fulFillItemPayload"] is not None:
            instance.ful_fill_item_payload = FulFillItemPayload.create_from_dict(
                dict_["fulFillItemPayload"], include_empty=include_empty
            )
        elif include_empty:
            instance.ful_fill_item_payload = FulFillItemPayload()
        if (
            "revokeEntitlementPayload" in dict_
            and dict_["revokeEntitlementPayload"] is not None
        ):
            instance.revoke_entitlement_payload = (
                RevokeEntitlementPayload.create_from_dict(
                    dict_["revokeEntitlementPayload"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.revoke_entitlement_payload = RevokeEntitlementPayload()
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = Union[str, TypeEnum]()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, OperationRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[OperationRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[OperationRequest, List[OperationRequest], Dict[Any, OperationRequest]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "creditPayload": "credit_payload",
            "debitPayload": "debit_payload",
            "fulFillItemPayload": "ful_fill_item_payload",
            "revokeEntitlementPayload": "revoke_entitlement_payload",
            "type": "type_",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "creditPayload": False,
            "debitPayload": False,
            "fulFillItemPayload": False,
            "revokeEntitlementPayload": False,
            "type": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "type": [
                "CREDIT_WALLET",
                "DEBIT_WALLET",
                "FULFILL_ITEM",
                "REVOKE_ENTITLEMENT",
            ],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class AdminEntitlementSoldRequest(Model):
    """A DTO object for sell entitlement. (AdminEntitlementSoldRequest)

    Properties:
        platform: (platform) OPTIONAL str

        request_id: (requestId) OPTIONAL str

        use_count: (useCount) OPTIONAL int
    """

    # region fields

    platform: str  # OPTIONAL
    request_id: str  # OPTIONAL
    use_count: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_platform(self, value: str) -> AdminEntitlementSoldRequest:
        self.platform = value
        return self

    def with_request_id(self, value: str) -> AdminEntitlementSoldRequest:
        self.request_id = value
        return self

    def with_use_count(self, value: int) -> AdminEntitlementSoldRequest:
        self.use_count = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "platform"):
            result["platform"] = str(self.platform)
        elif include_empty:
            result["platform"] = ""
        if hasattr(self, "request_id"):
            result["requestId"] = str(self.request_id)
        elif include_empty:
            result["requestId"] = ""
        if hasattr(self, "use_count"):
            result["useCount"] = int(self.use_count)
        elif include_empty:
            result["useCount"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        platform: Optional[str] = None,
        request_id: Optional[str] = None,
        use_count: Optional[int] = None,
        **kwargs,
    ) -> AdminEntitlementSoldRequest:
        instance = cls()
        if platform is not None:
            instance.platform = platform
        if request_id is not None:
            instance.request_id = request_id
        if use_count is not None:
            instance.use_count = use_count
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> AdminEntitlementSoldRequest:
        instance = cls()
        if not dict_:
            return instance
        if "platform" in dict_ and dict_["platform"] is not None:
            instance.platform = str(dict_["platform"])
        elif include_empty:
            instance.platform = ""
        if "requestId" in dict_ and dict_["requestId"] is not None:
            instance.request_id = str(dict_["requestId"])
        elif include_empty:
            instance.request_id = ""
        if "useCount" in dict_ and dict_["useCount"] is not None:
            instance.use_count = int(dict_["useCount"])
        elif include_empty:
            instance.use_count = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, AdminEntitlementSoldRequest]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[AdminEntitlementSoldRequest]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        AdminEntitlementSoldRequest,
        List[AdminEntitlementSoldRequest],
        Dict[Any, AdminEntitlementSoldRequest],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "platform": "platform",
            "requestId": "request_id",
            "useCount": "use_count",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "platform": False,
            "requestId": False,
            "useCount": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum


class PlatformEnum(StrEnum):
    APPLE = "APPLE"
    EPICGAMES = "EPICGAMES"
    GOOGLE = "GOOGLE"
    OCULUS = "OCULUS"
    PLAYSTATION = "PLAYSTATION"
    STADIA = "STADIA"
    STEAM = "STEAM"
    TWITCH = "TWITCH"
    XBOX = "XBOX"


class StatusEnum(StrEnum):
    ACTIVE = "ACTIVE"
    BILLING_RETRY_PERIOD = "BILLING_RETRY_PERIOD"
    CANCELED = "CANCELED"
    EXPIRED = "EXPIRED"
    IN_GRACE_PERIOD = "IN_GRACE_PERIOD"
    ON_HOLD = "ON_HOLD"
    PAUSED = "PAUSED"
    PENDING = "PENDING"
    REVOKED = "REVOKED"
    UNKNOWN = "UNKNOWN"


class ThirdPartyUserSubscriptionInfo(Model):
    """Third party user subscription info (ThirdPartyUserSubscriptionInfo)

    Properties:
        active: (active) REQUIRED bool

        expired_at: (expiredAt) REQUIRED str

        id_: (id) REQUIRED str

        namespace: (namespace) REQUIRED str

        platform: (platform) REQUIRED Union[str, PlatformEnum]

        status: (status) REQUIRED Union[str, StatusEnum]

        subscription_product_id: (subscriptionProductId) REQUIRED str

        user_id: (userId) REQUIRED str

        created_at: (createdAt) OPTIONAL str

        last_transaction_id: (lastTransactionId) OPTIONAL str

        start_at: (startAt) OPTIONAL str

        subscription_group_id: (subscriptionGroupId) OPTIONAL str

        updated_at: (updatedAt) OPTIONAL str
    """

    # region fields

    active: bool  # REQUIRED
    expired_at: str  # REQUIRED
    id_: str  # REQUIRED
    namespace: str  # REQUIRED
    platform: Union[str, PlatformEnum]  # REQUIRED
    status: Union[str, StatusEnum]  # REQUIRED
    subscription_product_id: str  # REQUIRED
    user_id: str  # REQUIRED
    created_at: str  # OPTIONAL
    last_transaction_id: str  # OPTIONAL
    start_at: str  # OPTIONAL
    subscription_group_id: str  # OPTIONAL
    updated_at: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_active(self, value: bool) -> ThirdPartyUserSubscriptionInfo:
        self.active = value
        return self

    def with_expired_at(self, value: str) -> ThirdPartyUserSubscriptionInfo:
        self.expired_at = value
        return self

    def with_id(self, value: str) -> ThirdPartyUserSubscriptionInfo:
        self.id_ = value
        return self

    def with_namespace(self, value: str) -> ThirdPartyUserSubscriptionInfo:
        self.namespace = value
        return self

    def with_platform(
        self, value: Union[str, PlatformEnum]
    ) -> ThirdPartyUserSubscriptionInfo:
        self.platform = value
        return self

    def with_status(
        self, value: Union[str, StatusEnum]
    ) -> ThirdPartyUserSubscriptionInfo:
        self.status = value
        return self

    def with_subscription_product_id(
        self, value: str
    ) -> ThirdPartyUserSubscriptionInfo:
        self.subscription_product_id = value
        return self

    def with_user_id(self, value: str) -> ThirdPartyUserSubscriptionInfo:
        self.user_id = value
        return self

    def with_created_at(self, value: str) -> ThirdPartyUserSubscriptionInfo:
        self.created_at = value
        return self

    def with_last_transaction_id(self, value: str) -> ThirdPartyUserSubscriptionInfo:
        self.last_transaction_id = value
        return self

    def with_start_at(self, value: str) -> ThirdPartyUserSubscriptionInfo:
        self.start_at = value
        return self

    def with_subscription_group_id(self, value: str) -> ThirdPartyUserSubscriptionInfo:
        self.subscription_group_id = value
        return self

    def with_updated_at(self, value: str) -> ThirdPartyUserSubscriptionInfo:
        self.updated_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "active"):
            result["active"] = bool(self.active)
        elif include_empty:
            result["active"] = False
        if hasattr(self, "expired_at"):
            result["expiredAt"] = str(self.expired_at)
        elif include_empty:
            result["expiredAt"] = ""
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "platform"):
            result["platform"] = str(self.platform)
        elif include_empty:
            result["platform"] = Union[str, PlatformEnum]()
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "subscription_product_id"):
            result["subscriptionProductId"] = str(self.subscription_product_id)
        elif include_empty:
            result["subscriptionProductId"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "last_transaction_id"):
            result["lastTransactionId"] = str(self.last_transaction_id)
        elif include_empty:
            result["lastTransactionId"] = ""
        if hasattr(self, "start_at"):
            result["startAt"] = str(self.start_at)
        elif include_empty:
            result["startAt"] = ""
        if hasattr(self, "subscription_group_id"):
            result["subscriptionGroupId"] = str(self.subscription_group_id)
        elif include_empty:
            result["subscriptionGroupId"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        active: bool,
        expired_at: str,
        id_: str,
        namespace: str,
        platform: Union[str, PlatformEnum],
        status: Union[str, StatusEnum],
        subscription_product_id: str,
        user_id: str,
        created_at: Optional[str] = None,
        last_transaction_id: Optional[str] = None,
        start_at: Optional[str] = None,
        subscription_group_id: Optional[str] = None,
        updated_at: Optional[str] = None,
        **kwargs,
    ) -> ThirdPartyUserSubscriptionInfo:
        instance = cls()
        instance.active = active
        instance.expired_at = expired_at
        instance.id_ = id_
        instance.namespace = namespace
        instance.platform = platform
        instance.status = status
        instance.subscription_product_id = subscription_product_id
        instance.user_id = user_id
        if created_at is not None:
            instance.created_at = created_at
        if last_transaction_id is not None:
            instance.last_transaction_id = last_transaction_id
        if start_at is not None:
            instance.start_at = start_at
        if subscription_group_id is not None:
            instance.subscription_group_id = subscription_group_id
        if updated_at is not None:
            instance.updated_at = updated_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ThirdPartyUserSubscriptionInfo:
        instance = cls()
        if not dict_:
            return instance
        if "active" in dict_ and dict_["active"] is not None:
            instance.active = bool(dict_["active"])
        elif include_empty:
            instance.active = False
        if "expiredAt" in dict_ and dict_["expiredAt"] is not None:
            instance.expired_at = str(dict_["expiredAt"])
        elif include_empty:
            instance.expired_at = ""
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "platform" in dict_ and dict_["platform"] is not None:
            instance.platform = str(dict_["platform"])
        elif include_empty:
            instance.platform = Union[str, PlatformEnum]()
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if (
            "subscriptionProductId" in dict_
            and dict_["subscriptionProductId"] is not None
        ):
            instance.subscription_product_id = str(dict_["subscriptionProductId"])
        elif include_empty:
            instance.subscription_product_id = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "lastTransactionId" in dict_ and dict_["lastTransactionId"] is not None:
            instance.last_transaction_id = str(dict_["lastTransactionId"])
        elif include_empty:
            instance.last_transaction_id = ""
        if "startAt" in dict_ and dict_["startAt"] is not None:
            instance.start_at = str(dict_["startAt"])
        elif include_empty:
            instance.start_at = ""
        if "subscriptionGroupId" in dict_ and dict_["subscriptionGroupId"] is not None:
            instance.subscription_group_id = str(dict_["subscriptionGroupId"])
        elif include_empty:
            instance.subscription_group_id = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ThirdPartyUserSubscriptionInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ThirdPartyUserSubscriptionInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ThirdPartyUserSubscriptionInfo,
        List[ThirdPartyUserSubscriptionInfo],
        Dict[Any, ThirdPartyUserSubscriptionInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "active": "active",
            "expiredAt": "expired_at",
            "id": "id_",
            "namespace": "namespace",
            "platform": "platform",
            "status": "status",
            "subscriptionProductId": "subscription_product_id",
            "userId": "user_id",
            "createdAt": "created_at",
            "lastTransactionId": "last_transaction_id",
            "startAt": "start_at",
            "subscriptionGroupId": "subscription_group_id",
            "updatedAt": "updated_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "active": True,
            "expiredAt": True,
            "id": True,
            "namespace": True,
            "platform": True,
            "status": True,
            "subscriptionProductId": True,
            "userId": True,
            "createdAt": False,
            "lastTransactionId": False,
            "startAt": False,
            "subscriptionGroupId": False,
            "updatedAt": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "platform": [
                "APPLE",
                "EPICGAMES",
                "GOOGLE",
                "OCULUS",
                "PLAYSTATION",
                "STADIA",
                "STEAM",
                "TWITCH",
                "XBOX",
            ],
            "status": [
                "ACTIVE",
                "BILLING_RETRY_PERIOD",
                "CANCELED",
                "EXPIRED",
                "IN_GRACE_PERIOD",
                "ON_HOLD",
                "PAUSED",
                "PENDING",
                "REVOKED",
                "UNKNOWN",
            ],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model

from ..models.stream_event_body import StreamEventBody


class StreamEvent(Model):
    """Stream event (StreamEvent)

    Properties:
        body: (body) OPTIONAL StreamEventBody

        event_domain: (eventDomain) OPTIONAL str

        event_source: (eventSource) OPTIONAL str

        event_type: (eventType) OPTIONAL str

        event_version: (eventVersion) OPTIONAL int

        id_: (id) OPTIONAL str

        timestamp: (timestamp) OPTIONAL str
    """

    # region fields

    body: StreamEventBody  # OPTIONAL
    event_domain: str  # OPTIONAL
    event_source: str  # OPTIONAL
    event_type: str  # OPTIONAL
    event_version: int  # OPTIONAL
    id_: str  # OPTIONAL
    timestamp: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_body(self, value: StreamEventBody) -> StreamEvent:
        self.body = value
        return self

    def with_event_domain(self, value: str) -> StreamEvent:
        self.event_domain = value
        return self

    def with_event_source(self, value: str) -> StreamEvent:
        self.event_source = value
        return self

    def with_event_type(self, value: str) -> StreamEvent:
        self.event_type = value
        return self

    def with_event_version(self, value: int) -> StreamEvent:
        self.event_version = value
        return self

    def with_id(self, value: str) -> StreamEvent:
        self.id_ = value
        return self

    def with_timestamp(self, value: str) -> StreamEvent:
        self.timestamp = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "body"):
            result["body"] = self.body.to_dict(include_empty=include_empty)
        elif include_empty:
            result["body"] = StreamEventBody()
        if hasattr(self, "event_domain"):
            result["eventDomain"] = str(self.event_domain)
        elif include_empty:
            result["eventDomain"] = ""
        if hasattr(self, "event_source"):
            result["eventSource"] = str(self.event_source)
        elif include_empty:
            result["eventSource"] = ""
        if hasattr(self, "event_type"):
            result["eventType"] = str(self.event_type)
        elif include_empty:
            result["eventType"] = ""
        if hasattr(self, "event_version"):
            result["eventVersion"] = int(self.event_version)
        elif include_empty:
            result["eventVersion"] = 0
        if hasattr(self, "id_"):
            result["id"] = str(self.id_)
        elif include_empty:
            result["id"] = ""
        if hasattr(self, "timestamp"):
            result["timestamp"] = str(self.timestamp)
        elif include_empty:
            result["timestamp"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        body: Optional[StreamEventBody] = None,
        event_domain: Optional[str] = None,
        event_source: Optional[str] = None,
        event_type: Optional[str] = None,
        event_version: Optional[int] = None,
        id_: Optional[str] = None,
        timestamp: Optional[str] = None,
        **kwargs,
    ) -> StreamEvent:
        instance = cls()
        if body is not None:
            instance.body = body
        if event_domain is not None:
            instance.event_domain = event_domain
        if event_source is not None:
            instance.event_source = event_source
        if event_type is not None:
            instance.event_type = event_type
        if event_version is not None:
            instance.event_version = event_version
        if id_ is not None:
            instance.id_ = id_
        if timestamp is not None:
            instance.timestamp = timestamp
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> StreamEvent:
        instance = cls()
        if not dict_:
            return instance
        if "body" in dict_ and dict_["body"] is not None:
            instance.body = StreamEventBody.create_from_dict(
                dict_["body"], include_empty=include_empty
            )
        elif include_empty:
            instance.body = StreamEventBody()
        if "eventDomain" in dict_ and dict_["eventDomain"] is not None:
            instance.event_domain = str(dict_["eventDomain"])
        elif include_empty:
            instance.event_domain = ""
        if "eventSource" in dict_ and dict_["eventSource"] is not None:
            instance.event_source = str(dict_["eventSource"])
        elif include_empty:
            instance.event_source = ""
        if "eventType" in dict_ and dict_["eventType"] is not None:
            instance.event_type = str(dict_["eventType"])
        elif include_empty:
            instance.event_type = ""
        if "eventVersion" in dict_ and dict_["eventVersion"] is not None:
            instance.event_version = int(dict_["eventVersion"])
        elif include_empty:
            instance.event_version = 0
        if "id" in dict_ and dict_["id"] is not None:
            instance.id_ = str(dict_["id"])
        elif include_empty:
            instance.id_ = ""
        if "timestamp" in dict_ and dict_["timestamp"] is not None:
            instance.timestamp = str(dict_["timestamp"])
        elif include_empty:
            instance.timestamp = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, StreamEvent]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[StreamEvent]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[StreamEvent, List[StreamEvent], Dict[Any, StreamEvent]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "body": "body",
            "eventDomain": "event_domain",
            "eventSource": "event_source",
            "eventType": "event_type",
            "eventVersion": "event_version",
            "id": "id_",
            "timestamp": "timestamp",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "body": False,
            "eventDomain": False,
            "eventSource": False,
            "eventType": False,
            "eventVersion": False,
            "id": False,
            "timestamp": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum

from ..models.discount_code_deduction_detail import DiscountCodeDeductionDetail


class DeductionTypeEnum(StrEnum):
    DISCOUNT_CODE = "DISCOUNT_CODE"
    PAYMENT_DISCOUNT_CODE = "PAYMENT_DISCOUNT_CODE"


class DeductionDetail(Model):
    """Deduction detail (DeductionDetail)

    Properties:
        deduction_type: (deductionType) OPTIONAL Union[str, DeductionTypeEnum]

        discount_code_deduction_detail: (discountCodeDeductionDetail) OPTIONAL DiscountCodeDeductionDetail

        discount_provider_name: (discountProviderName) OPTIONAL str
    """

    # region fields

    deduction_type: Union[str, DeductionTypeEnum]  # OPTIONAL
    discount_code_deduction_detail: DiscountCodeDeductionDetail  # OPTIONAL
    discount_provider_name: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_deduction_type(
        self, value: Union[str, DeductionTypeEnum]
    ) -> DeductionDetail:
        self.deduction_type = value
        return self

    def with_discount_code_deduction_detail(
        self, value: DiscountCodeDeductionDetail
    ) -> DeductionDetail:
        self.discount_code_deduction_detail = value
        return self

    def with_discount_provider_name(self, value: str) -> DeductionDetail:
        self.discount_provider_name = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "deduction_type"):
            result["deductionType"] = str(self.deduction_type)
        elif include_empty:
            result["deductionType"] = Union[str, DeductionTypeEnum]()
        if hasattr(self, "discount_code_deduction_detail"):
            result[
                "discountCodeDeductionDetail"
            ] = self.discount_code_deduction_detail.to_dict(include_empty=include_empty)
        elif include_empty:
            result["discountCodeDeductionDetail"] = DiscountCodeDeductionDetail()
        if hasattr(self, "discount_provider_name"):
            result["discountProviderName"] = str(self.discount_provider_name)
        elif include_empty:
            result["discountProviderName"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        deduction_type: Optional[Union[str, DeductionTypeEnum]] = None,
        discount_code_deduction_detail: Optional[DiscountCodeDeductionDetail] = None,
        discount_provider_name: Optional[str] = None,
        **kwargs,
    ) -> DeductionDetail:
        instance = cls()
        if deduction_type is not None:
            instance.deduction_type = deduction_type
        if discount_code_deduction_detail is not None:
            instance.discount_code_deduction_detail = discount_code_deduction_detail
        if discount_provider_name is not None:
            instance.discount_provider_name = discount_provider_name
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> DeductionDetail:
        instance = cls()
        if not dict_:
            return instance
        if "deductionType" in dict_ and dict_["deductionType"] is not None:
            instance.deduction_type = str(dict_["deductionType"])
        elif include_empty:
            instance.deduction_type = Union[str, DeductionTypeEnum]()
        if (
            "discountCodeDeductionDetail" in dict_
            and dict_["discountCodeDeductionDetail"] is not None
        ):
            instance.discount_code_deduction_detail = (
                DiscountCodeDeductionDetail.create_from_dict(
                    dict_["discountCodeDeductionDetail"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.discount_code_deduction_detail = DiscountCodeDeductionDetail()
        if (
            "discountProviderName" in dict_
            and dict_["discountProviderName"] is not None
        ):
            instance.discount_provider_name = str(dict_["discountProviderName"])
        elif include_empty:
            instance.discount_provider_name = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, DeductionDetail]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[DeductionDetail]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[DeductionDetail, List[DeductionDetail], Dict[Any, DeductionDetail]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "deductionType": "deduction_type",
            "discountCodeDeductionDetail": "discount_code_deduction_detail",
            "discountProviderName": "discount_provider_name",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "deductionType": False,
            "discountCodeDeductionDetail": False,
            "discountProviderName": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "deductionType": ["DISCOUNT_CODE", "PAYMENT_DISCOUNT_CODE"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model

from ..models.reward_condition import RewardCondition


class RewardCreate(Model):
    """Reward create (RewardCreate)

    Properties:
        event_topic: (eventTopic) REQUIRED str

        reward_code: (rewardCode) REQUIRED str

        description: (description) OPTIONAL str

        max_awarded: (maxAwarded) OPTIONAL int

        max_awarded_per_user: (maxAwardedPerUser) OPTIONAL int

        namespace_expression: (namespaceExpression) OPTIONAL str

        reward_conditions: (rewardConditions) OPTIONAL List[RewardCondition]

        user_id_expression: (userIdExpression) OPTIONAL str
    """

    # region fields

    event_topic: str  # REQUIRED
    reward_code: str  # REQUIRED
    description: str  # OPTIONAL
    max_awarded: int  # OPTIONAL
    max_awarded_per_user: int  # OPTIONAL
    namespace_expression: str  # OPTIONAL
    reward_conditions: List[RewardCondition]  # OPTIONAL
    user_id_expression: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_event_topic(self, value: str) -> RewardCreate:
        self.event_topic = value
        return self

    def with_reward_code(self, value: str) -> RewardCreate:
        self.reward_code = value
        return self

    def with_description(self, value: str) -> RewardCreate:
        self.description = value
        return self

    def with_max_awarded(self, value: int) -> RewardCreate:
        self.max_awarded = value
        return self

    def with_max_awarded_per_user(self, value: int) -> RewardCreate:
        self.max_awarded_per_user = value
        return self

    def with_namespace_expression(self, value: str) -> RewardCreate:
        self.namespace_expression = value
        return self

    def with_reward_conditions(self, value: List[RewardCondition]) -> RewardCreate:
        self.reward_conditions = value
        return self

    def with_user_id_expression(self, value: str) -> RewardCreate:
        self.user_id_expression = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "event_topic"):
            result["eventTopic"] = str(self.event_topic)
        elif include_empty:
            result["eventTopic"] = ""
        if hasattr(self, "reward_code"):
            result["rewardCode"] = str(self.reward_code)
        elif include_empty:
            result["rewardCode"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "max_awarded"):
            result["maxAwarded"] = int(self.max_awarded)
        elif include_empty:
            result["maxAwarded"] = 0
        if hasattr(self, "max_awarded_per_user"):
            result["maxAwardedPerUser"] = int(self.max_awarded_per_user)
        elif include_empty:
            result["maxAwardedPerUser"] = 0
        if hasattr(self, "namespace_expression"):
            result["namespaceExpression"] = str(self.namespace_expression)
        elif include_empty:
            result["namespaceExpression"] = ""
        if hasattr(self, "reward_conditions"):
            result["rewardConditions"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.reward_conditions
            ]
        elif include_empty:
            result["rewardConditions"] = []
        if hasattr(self, "user_id_expression"):
            result["userIdExpression"] = str(self.user_id_expression)
        elif include_empty:
            result["userIdExpression"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        event_topic: str,
        reward_code: str,
        description: Optional[str] = None,
        max_awarded: Optional[int] = None,
        max_awarded_per_user: Optional[int] = None,
        namespace_expression: Optional[str] = None,
        reward_conditions: Optional[List[RewardCondition]] = None,
        user_id_expression: Optional[str] = None,
        **kwargs,
    ) -> RewardCreate:
        instance = cls()
        instance.event_topic = event_topic
        instance.reward_code = reward_code
        if description is not None:
            instance.description = description
        if max_awarded is not None:
            instance.max_awarded = max_awarded
        if max_awarded_per_user is not None:
            instance.max_awarded_per_user = max_awarded_per_user
        if namespace_expression is not None:
            instance.namespace_expression = namespace_expression
        if reward_conditions is not None:
            instance.reward_conditions = reward_conditions
        if user_id_expression is not None:
            instance.user_id_expression = user_id_expression
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> RewardCreate:
        instance = cls()
        if not dict_:
            return instance
        if "eventTopic" in dict_ and dict_["eventTopic"] is not None:
            instance.event_topic = str(dict_["eventTopic"])
        elif include_empty:
            instance.event_topic = ""
        if "rewardCode" in dict_ and dict_["rewardCode"] is not None:
            instance.reward_code = str(dict_["rewardCode"])
        elif include_empty:
            instance.reward_code = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "maxAwarded" in dict_ and dict_["maxAwarded"] is not None:
            instance.max_awarded = int(dict_["maxAwarded"])
        elif include_empty:
            instance.max_awarded = 0
        if "maxAwardedPerUser" in dict_ and dict_["maxAwardedPerUser"] is not None:
            instance.max_awarded_per_user = int(dict_["maxAwardedPerUser"])
        elif include_empty:
            instance.max_awarded_per_user = 0
        if "namespaceExpression" in dict_ and dict_["namespaceExpression"] is not None:
            instance.namespace_expression = str(dict_["namespaceExpression"])
        elif include_empty:
            instance.namespace_expression = ""
        if "rewardConditions" in dict_ and dict_["rewardConditions"] is not None:
            instance.reward_conditions = [
                RewardCondition.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["rewardConditions"]
            ]
        elif include_empty:
            instance.reward_conditions = []
        if "userIdExpression" in dict_ and dict_["userIdExpression"] is not None:
            instance.user_id_expression = str(dict_["userIdExpression"])
        elif include_empty:
            instance.user_id_expression = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, RewardCreate]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[RewardCreate]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[RewardCreate, List[RewardCreate], Dict[Any, RewardCreate]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "eventTopic": "event_topic",
            "rewardCode": "reward_code",
            "description": "description",
            "maxAwarded": "max_awarded",
            "maxAwardedPerUser": "max_awarded_per_user",
            "namespaceExpression": "namespace_expression",
            "rewardConditions": "reward_conditions",
            "userIdExpression": "user_id_expression",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "eventTopic": True,
            "rewardCode": True,
            "description": False,
            "maxAwarded": False,
            "maxAwardedPerUser": False,
            "namespaceExpression": False,
            "rewardConditions": False,
            "userIdExpression": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum

from ..models.action import Action


class StatusEnum(StrEnum):
    FAILED = "FAILED"
    INIT = "INIT"
    SUCCESS = "SUCCESS"


class TradeChainActionHistoryInfo(Model):
    """Trade chain action history info (TradeChainActionHistoryInfo)

    Properties:
        actions: (actions) OPTIONAL List[Action]

        metadata: (metadata) OPTIONAL Dict[str, Any]

        namespace: (namespace) OPTIONAL str

        status: (status) OPTIONAL Union[str, StatusEnum]

        status_reason: (statusReason) OPTIONAL str

        transaction_id: (transactionId) OPTIONAL str

        type_: (type) OPTIONAL str

        user_ids: (userIds) OPTIONAL List[str]
    """

    # region fields

    actions: List[Action]  # OPTIONAL
    metadata: Dict[str, Any]  # OPTIONAL
    namespace: str  # OPTIONAL
    status: Union[str, StatusEnum]  # OPTIONAL
    status_reason: str  # OPTIONAL
    transaction_id: str  # OPTIONAL
    type_: str  # OPTIONAL
    user_ids: List[str]  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_actions(self, value: List[Action]) -> TradeChainActionHistoryInfo:
        self.actions = value
        return self

    def with_metadata(self, value: Dict[str, Any]) -> TradeChainActionHistoryInfo:
        self.metadata = value
        return self

    def with_namespace(self, value: str) -> TradeChainActionHistoryInfo:
        self.namespace = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> TradeChainActionHistoryInfo:
        self.status = value
        return self

    def with_status_reason(self, value: str) -> TradeChainActionHistoryInfo:
        self.status_reason = value
        return self

    def with_transaction_id(self, value: str) -> TradeChainActionHistoryInfo:
        self.transaction_id = value
        return self

    def with_type(self, value: str) -> TradeChainActionHistoryInfo:
        self.type_ = value
        return self

    def with_user_ids(self, value: List[str]) -> TradeChainActionHistoryInfo:
        self.user_ids = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "actions"):
            result["actions"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.actions
            ]
        elif include_empty:
            result["actions"] = []
        if hasattr(self, "metadata"):
            result["metadata"] = {str(k0): v0 for k0, v0 in self.metadata.items()}
        elif include_empty:
            result["metadata"] = {}
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "status_reason"):
            result["statusReason"] = str(self.status_reason)
        elif include_empty:
            result["statusReason"] = ""
        if hasattr(self, "transaction_id"):
            result["transactionId"] = str(self.transaction_id)
        elif include_empty:
            result["transactionId"] = ""
        if hasattr(self, "type_"):
            result["type"] = str(self.type_)
        elif include_empty:
            result["type"] = ""
        if hasattr(self, "user_ids"):
            result["userIds"] = [str(i0) for i0 in self.user_ids]
        elif include_empty:
            result["userIds"] = []
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        actions: Optional[List[Action]] = None,
        metadata: Optional[Dict[str, Any]] = None,
        namespace: Optional[str] = None,
        status: Optional[Union[str, StatusEnum]] = None,
        status_reason: Optional[str] = None,
        transaction_id: Optional[str] = None,
        type_: Optional[str] = None,
        user_ids: Optional[List[str]] = None,
        **kwargs,
    ) -> TradeChainActionHistoryInfo:
        instance = cls()
        if actions is not None:
            instance.actions = actions
        if metadata is not None:
            instance.metadata = metadata
        if namespace is not None:
            instance.namespace = namespace
        if status is not None:
            instance.status = status
        if status_reason is not None:
            instance.status_reason = status_reason
        if transaction_id is not None:
            instance.transaction_id = transaction_id
        if type_ is not None:
            instance.type_ = type_
        if user_ids is not None:
            instance.user_ids = user_ids
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> TradeChainActionHistoryInfo:
        instance = cls()
        if not dict_:
            return instance
        if "actions" in dict_ and dict_["actions"] is not None:
            instance.actions = [
                Action.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["actions"]
            ]
        elif include_empty:
            instance.actions = []
        if "metadata" in dict_ and dict_["metadata"] is not None:
            instance.metadata = {str(k0): v0 for k0, v0 in dict_["metadata"].items()}
        elif include_empty:
            instance.metadata = {}
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "statusReason" in dict_ and dict_["statusReason"] is not None:
            instance.status_reason = str(dict_["statusReason"])
        elif include_empty:
            instance.status_reason = ""
        if "transactionId" in dict_ and dict_["transactionId"] is not None:
            instance.transaction_id = str(dict_["transactionId"])
        elif include_empty:
            instance.transaction_id = ""
        if "type" in dict_ and dict_["type"] is not None:
            instance.type_ = str(dict_["type"])
        elif include_empty:
            instance.type_ = ""
        if "userIds" in dict_ and dict_["userIds"] is not None:
            instance.user_ids = [str(i0) for i0 in dict_["userIds"]]
        elif include_empty:
            instance.user_ids = []
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, TradeChainActionHistoryInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[TradeChainActionHistoryInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        TradeChainActionHistoryInfo,
        List[TradeChainActionHistoryInfo],
        Dict[Any, TradeChainActionHistoryInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "actions": "actions",
            "metadata": "metadata",
            "namespace": "namespace",
            "status": "status",
            "statusReason": "status_reason",
            "transactionId": "transaction_id",
            "type": "type_",
            "userIds": "user_ids",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "actions": False,
            "metadata": False,
            "namespace": False,
            "status": False,
            "statusReason": False,
            "transactionId": False,
            "type": False,
            "userIds": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["FAILED", "INIT", "SUCCESS"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum


class StatusEnum(StrEnum):
    FAILED = "FAILED"
    FULFILLED = "FULFILLED"
    PARTIAL_REVOKED = "PARTIAL_REVOKED"
    REVOKED = "REVOKED"
    VERIFIED = "VERIFIED"


class EpicGamesReconcileResult(Model):
    """Epic games reconcile result (EpicGamesReconcileResult)

    Properties:
        epic_games_item_id: (epicGamesItemId) OPTIONAL str

        item_id: (itemId) OPTIONAL str

        sku: (sku) OPTIONAL str

        status: (status) OPTIONAL Union[str, StatusEnum]

        transaction_id: (transactionId) OPTIONAL str
    """

    # region fields

    epic_games_item_id: str  # OPTIONAL
    item_id: str  # OPTIONAL
    sku: str  # OPTIONAL
    status: Union[str, StatusEnum]  # OPTIONAL
    transaction_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_epic_games_item_id(self, value: str) -> EpicGamesReconcileResult:
        self.epic_games_item_id = value
        return self

    def with_item_id(self, value: str) -> EpicGamesReconcileResult:
        self.item_id = value
        return self

    def with_sku(self, value: str) -> EpicGamesReconcileResult:
        self.sku = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> EpicGamesReconcileResult:
        self.status = value
        return self

    def with_transaction_id(self, value: str) -> EpicGamesReconcileResult:
        self.transaction_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "epic_games_item_id"):
            result["epicGamesItemId"] = str(self.epic_games_item_id)
        elif include_empty:
            result["epicGamesItemId"] = ""
        if hasattr(self, "item_id"):
            result["itemId"] = str(self.item_id)
        elif include_empty:
            result["itemId"] = ""
        if hasattr(self, "sku"):
            result["sku"] = str(self.sku)
        elif include_empty:
            result["sku"] = ""
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "transaction_id"):
            result["transactionId"] = str(self.transaction_id)
        elif include_empty:
            result["transactionId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        epic_games_item_id: Optional[str] = None,
        item_id: Optional[str] = None,
        sku: Optional[str] = None,
        status: Optional[Union[str, StatusEnum]] = None,
        transaction_id: Optional[str] = None,
        **kwargs,
    ) -> EpicGamesReconcileResult:
        instance = cls()
        if epic_games_item_id is not None:
            instance.epic_games_item_id = epic_games_item_id
        if item_id is not None:
            instance.item_id = item_id
        if sku is not None:
            instance.sku = sku
        if status is not None:
            instance.status = status
        if transaction_id is not None:
            instance.transaction_id = transaction_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> EpicGamesReconcileResult:
        instance = cls()
        if not dict_:
            return instance
        if "epicGamesItemId" in dict_ and dict_["epicGamesItemId"] is not None:
            instance.epic_games_item_id = str(dict_["epicGamesItemId"])
        elif include_empty:
            instance.epic_games_item_id = ""
        if "itemId" in dict_ and dict_["itemId"] is not None:
            instance.item_id = str(dict_["itemId"])
        elif include_empty:
            instance.item_id = ""
        if "sku" in dict_ and dict_["sku"] is not None:
            instance.sku = str(dict_["sku"])
        elif include_empty:
            instance.sku = ""
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "transactionId" in dict_ and dict_["transactionId"] is not None:
            instance.transaction_id = str(dict_["transactionId"])
        elif include_empty:
            instance.transaction_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, EpicGamesReconcileResult]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[EpicGamesReconcileResult]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        EpicGamesReconcileResult,
        List[EpicGamesReconcileResult],
        Dict[Any, EpicGamesReconcileResult],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "epicGamesItemId": "epic_games_item_id",
            "itemId": "item_id",
            "sku": "sku",
            "status": "status",
            "transactionId": "transaction_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "epicGamesItemId": False,
            "itemId": False,
            "sku": False,
            "status": False,
            "transactionId": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["FAILED", "FULFILLED", "PARTIAL_REVOKED", "REVOKED", "VERIFIED"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum

from ..models.operation import Operation


class StatusEnum(StrEnum):
    FAILED = "FAILED"
    INIT = "INIT"
    SKIPPED = "SKIPPED"
    SUCCESS = "SUCCESS"


class Action(Model):
    """Action (Action)

    Properties:
        operations: (operations) OPTIONAL List[Operation]

        status: (status) OPTIONAL Union[str, StatusEnum]

        status_reason: (statusReason) OPTIONAL str

        user_id: (userId) OPTIONAL str
    """

    # region fields

    operations: List[Operation]  # OPTIONAL
    status: Union[str, StatusEnum]  # OPTIONAL
    status_reason: str  # OPTIONAL
    user_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_operations(self, value: List[Operation]) -> Action:
        self.operations = value
        return self

    def with_status(self, value: Union[str, StatusEnum]) -> Action:
        self.status = value
        return self

    def with_status_reason(self, value: str) -> Action:
        self.status_reason = value
        return self

    def with_user_id(self, value: str) -> Action:
        self.user_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "operations"):
            result["operations"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.operations
            ]
        elif include_empty:
            result["operations"] = []
        if hasattr(self, "status"):
            result["status"] = str(self.status)
        elif include_empty:
            result["status"] = Union[str, StatusEnum]()
        if hasattr(self, "status_reason"):
            result["statusReason"] = str(self.status_reason)
        elif include_empty:
            result["statusReason"] = ""
        if hasattr(self, "user_id"):
            result["userId"] = str(self.user_id)
        elif include_empty:
            result["userId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        operations: Optional[List[Operation]] = None,
        status: Optional[Union[str, StatusEnum]] = None,
        status_reason: Optional[str] = None,
        user_id: Optional[str] = None,
        **kwargs,
    ) -> Action:
        instance = cls()
        if operations is not None:
            instance.operations = operations
        if status is not None:
            instance.status = status
        if status_reason is not None:
            instance.status_reason = status_reason
        if user_id is not None:
            instance.user_id = user_id
        return instance

    @classmethod
    def create_from_dict(cls, dict_: dict, include_empty: bool = False) -> Action:
        instance = cls()
        if not dict_:
            return instance
        if "operations" in dict_ and dict_["operations"] is not None:
            instance.operations = [
                Operation.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["operations"]
            ]
        elif include_empty:
            instance.operations = []
        if "status" in dict_ and dict_["status"] is not None:
            instance.status = str(dict_["status"])
        elif include_empty:
            instance.status = Union[str, StatusEnum]()
        if "statusReason" in dict_ and dict_["statusReason"] is not None:
            instance.status_reason = str(dict_["statusReason"])
        elif include_empty:
            instance.status_reason = ""
        if "userId" in dict_ and dict_["userId"] is not None:
            instance.user_id = str(dict_["userId"])
        elif include_empty:
            instance.user_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, Action]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[Action]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[Action, List[Action], Dict[Any, Action]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "operations": "operations",
            "status": "status",
            "statusReason": "status_reason",
            "userId": "user_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "operations": False,
            "status": False,
            "statusReason": False,
            "userId": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "status": ["FAILED", "INIT", "SKIPPED", "SUCCESS"],
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model

from ..models.xbl_clawback_event_subscription_data import (
    XblClawbackEventSubscriptionData,
)


class XblClawbackContractV2(Model):
    """Xbl clawback contract V2 (XblClawbackContractV2)

    Properties:
        event_state: (eventState) REQUIRED str

        line_item_id: (lineItemId) REQUIRED str

        order_id: (orderId) REQUIRED str

        product_id: (productId) REQUIRED str

        event_date: (eventDate) OPTIONAL str

        product_type: (productType) OPTIONAL str

        purchased_date: (purchasedDate) OPTIONAL str

        sandbox_id: (sandboxId) OPTIONAL str

        sku_id: (skuId) OPTIONAL str

        subscription_data: (subscriptionData) OPTIONAL XblClawbackEventSubscriptionData
    """

    # region fields

    event_state: str  # REQUIRED
    line_item_id: str  # REQUIRED
    order_id: str  # REQUIRED
    product_id: str  # REQUIRED
    event_date: str  # OPTIONAL
    product_type: str  # OPTIONAL
    purchased_date: str  # OPTIONAL
    sandbox_id: str  # OPTIONAL
    sku_id: str  # OPTIONAL
    subscription_data: XblClawbackEventSubscriptionData  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_event_state(self, value: str) -> XblClawbackContractV2:
        self.event_state = value
        return self

    def with_line_item_id(self, value: str) -> XblClawbackContractV2:
        self.line_item_id = value
        return self

    def with_order_id(self, value: str) -> XblClawbackContractV2:
        self.order_id = value
        return self

    def with_product_id(self, value: str) -> XblClawbackContractV2:
        self.product_id = value
        return self

    def with_event_date(self, value: str) -> XblClawbackContractV2:
        self.event_date = value
        return self

    def with_product_type(self, value: str) -> XblClawbackContractV2:
        self.product_type = value
        return self

    def with_purchased_date(self, value: str) -> XblClawbackContractV2:
        self.purchased_date = value
        return self

    def with_sandbox_id(self, value: str) -> XblClawbackContractV2:
        self.sandbox_id = value
        return self

    def with_sku_id(self, value: str) -> XblClawbackContractV2:
        self.sku_id = value
        return self

    def with_subscription_data(
        self, value: XblClawbackEventSubscriptionData
    ) -> XblClawbackContractV2:
        self.subscription_data = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "event_state"):
            result["eventState"] = str(self.event_state)
        elif include_empty:
            result["eventState"] = ""
        if hasattr(self, "line_item_id"):
            result["lineItemId"] = str(self.line_item_id)
        elif include_empty:
            result["lineItemId"] = ""
        if hasattr(self, "order_id"):
            result["orderId"] = str(self.order_id)
        elif include_empty:
            result["orderId"] = ""
        if hasattr(self, "product_id"):
            result["productId"] = str(self.product_id)
        elif include_empty:
            result["productId"] = ""
        if hasattr(self, "event_date"):
            result["eventDate"] = str(self.event_date)
        elif include_empty:
            result["eventDate"] = ""
        if hasattr(self, "product_type"):
            result["productType"] = str(self.product_type)
        elif include_empty:
            result["productType"] = ""
        if hasattr(self, "purchased_date"):
            result["purchasedDate"] = str(self.purchased_date)
        elif include_empty:
            result["purchasedDate"] = ""
        if hasattr(self, "sandbox_id"):
            result["sandboxId"] = str(self.sandbox_id)
        elif include_empty:
            result["sandboxId"] = ""
        if hasattr(self, "sku_id"):
            result["skuId"] = str(self.sku_id)
        elif include_empty:
            result["skuId"] = ""
        if hasattr(self, "subscription_data"):
            result["subscriptionData"] = self.subscription_data.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["subscriptionData"] = XblClawbackEventSubscriptionData()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        event_state: str,
        line_item_id: str,
        order_id: str,
        product_id: str,
        event_date: Optional[str] = None,
        product_type: Optional[str] = None,
        purchased_date: Optional[str] = None,
        sandbox_id: Optional[str] = None,
        sku_id: Optional[str] = None,
        subscription_data: Optional[XblClawbackEventSubscriptionData] = None,
        **kwargs,
    ) -> XblClawbackContractV2:
        instance = cls()
        instance.event_state = event_state
        instance.line_item_id = line_item_id
        instance.order_id = order_id
        instance.product_id = product_id
        if event_date is not None:
            instance.event_date = event_date
        if product_type is not None:
            instance.product_type = product_type
        if purchased_date is not None:
            instance.purchased_date = purchased_date
        if sandbox_id is not None:
            instance.sandbox_id = sandbox_id
        if sku_id is not None:
            instance.sku_id = sku_id
        if subscription_data is not None:
            instance.subscription_data = subscription_data
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> XblClawbackContractV2:
        instance = cls()
        if not dict_:
            return instance
        if "eventState" in dict_ and dict_["eventState"] is not None:
            instance.event_state = str(dict_["eventState"])
        elif include_empty:
            instance.event_state = ""
        if "lineItemId" in dict_ and dict_["lineItemId"] is not None:
            instance.line_item_id = str(dict_["lineItemId"])
        elif include_empty:
            instance.line_item_id = ""
        if "orderId" in dict_ and dict_["orderId"] is not None:
            instance.order_id = str(dict_["orderId"])
        elif include_empty:
            instance.order_id = ""
        if "productId" in dict_ and dict_["productId"] is not None:
            instance.product_id = str(dict_["productId"])
        elif include_empty:
            instance.product_id = ""
        if "eventDate" in dict_ and dict_["eventDate"] is not None:
            instance.event_date = str(dict_["eventDate"])
        elif include_empty:
            instance.event_date = ""
        if "productType" in dict_ and dict_["productType"] is not None:
            instance.product_type = str(dict_["productType"])
        elif include_empty:
            instance.product_type = ""
        if "purchasedDate" in dict_ and dict_["purchasedDate"] is not None:
            instance.purchased_date = str(dict_["purchasedDate"])
        elif include_empty:
            instance.purchased_date = ""
        if "sandboxId" in dict_ and dict_["sandboxId"] is not None:
            instance.sandbox_id = str(dict_["sandboxId"])
        elif include_empty:
            instance.sandbox_id = ""
        if "skuId" in dict_ and dict_["skuId"] is not None:
            instance.sku_id = str(dict_["skuId"])
        elif include_empty:
            instance.sku_id = ""
        if "subscriptionData" in dict_ and dict_["subscriptionData"] is not None:
            instance.subscription_data = (
                XblClawbackEventSubscriptionData.create_from_dict(
                    dict_["subscriptionData"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.subscription_data = XblClawbackEventSubscriptionData()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, XblClawbackContractV2]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[XblClawbackContractV2]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        XblClawbackContractV2,
        List[XblClawbackContractV2],
        Dict[Any, XblClawbackContractV2],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "eventState": "event_state",
            "lineItemId": "line_item_id",
            "orderId": "order_id",
            "productId": "product_id",
            "eventDate": "event_date",
            "productType": "product_type",
            "purchasedDate": "purchased_date",
            "sandboxId": "sandbox_id",
            "skuId": "sku_id",
            "subscriptionData": "subscription_data",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "eventState": True,
            "lineItemId": True,
            "orderId": True,
            "productId": True,
            "eventDate": False,
            "productType": False,
            "purchasedDate": False,
            "sandboxId": False,
            "skuId": False,
            "subscriptionData": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class XblClawbackEventSubscriptionData(Model):
    """Xbl clawback event subscription data (XblClawbackEventSubscriptionData)

    Properties:
        consumed_duration_in_days: (consumedDurationInDays) OPTIONAL int

        date_time: (dateTime) OPTIONAL str

        duration_in_days: (durationInDays) OPTIONAL int

        recurrence_id: (recurrenceId) OPTIONAL str
    """

    # region fields

    consumed_duration_in_days: int  # OPTIONAL
    date_time: str  # OPTIONAL
    duration_in_days: int  # OPTIONAL
    recurrence_id: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_consumed_duration_in_days(
        self, value: int
    ) -> XblClawbackEventSubscriptionData:
        self.consumed_duration_in_days = value
        return self

    def with_date_time(self, value: str) -> XblClawbackEventSubscriptionData:
        self.date_time = value
        return self

    def with_duration_in_days(self, value: int) -> XblClawbackEventSubscriptionData:
        self.duration_in_days = value
        return self

    def with_recurrence_id(self, value: str) -> XblClawbackEventSubscriptionData:
        self.recurrence_id = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "consumed_duration_in_days"):
            result["consumedDurationInDays"] = int(self.consumed_duration_in_days)
        elif include_empty:
            result["consumedDurationInDays"] = 0
        if hasattr(self, "date_time"):
            result["dateTime"] = str(self.date_time)
        elif include_empty:
            result["dateTime"] = ""
        if hasattr(self, "duration_in_days"):
            result["durationInDays"] = int(self.duration_in_days)
        elif include_empty:
            result["durationInDays"] = 0
        if hasattr(self, "recurrence_id"):
            result["recurrenceId"] = str(self.recurrence_id)
        elif include_empty:
            result["recurrenceId"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        consumed_duration_in_days: Optional[int] = None,
        date_time: Optional[str] = None,
        duration_in_days: Optional[int] = None,
        recurrence_id: Optional[str] = None,
        **kwargs,
    ) -> XblClawbackEventSubscriptionData:
        instance = cls()
        if consumed_duration_in_days is not None:
            instance.consumed_duration_in_days = consumed_duration_in_days
        if date_time is not None:
            instance.date_time = date_time
        if duration_in_days is not None:
            instance.duration_in_days = duration_in_days
        if recurrence_id is not None:
            instance.recurrence_id = recurrence_id
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> XblClawbackEventSubscriptionData:
        instance = cls()
        if not dict_:
            return instance
        if (
            "consumedDurationInDays" in dict_
            and dict_["consumedDurationInDays"] is not None
        ):
            instance.consumed_duration_in_days = int(dict_["consumedDurationInDays"])
        elif include_empty:
            instance.consumed_duration_in_days = 0
        if "dateTime" in dict_ and dict_["dateTime"] is not None:
            instance.date_time = str(dict_["dateTime"])
        elif include_empty:
            instance.date_time = ""
        if "durationInDays" in dict_ and dict_["durationInDays"] is not None:
            instance.duration_in_days = int(dict_["durationInDays"])
        elif include_empty:
            instance.duration_in_days = 0
        if "recurrenceId" in dict_ and dict_["recurrenceId"] is not None:
            instance.recurrence_id = str(dict_["recurrenceId"])
        elif include_empty:
            instance.recurrence_id = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, XblClawbackEventSubscriptionData]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[XblClawbackEventSubscriptionData]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        XblClawbackEventSubscriptionData,
        List[XblClawbackEventSubscriptionData],
        Dict[Any, XblClawbackEventSubscriptionData],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "consumedDurationInDays": "consumed_duration_in_days",
            "dateTime": "date_time",
            "durationInDays": "duration_in_days",
            "recurrenceId": "recurrence_id",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "consumedDurationInDays": False,
            "dateTime": False,
            "durationInDays": False,
            "recurrenceId": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: wrapper.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import get_namespace as get_services_namespace
from accelbyte_py_sdk.core import run_request
from accelbyte_py_sdk.core import run_request_async
from accelbyte_py_sdk.core import same_doc_as

from ..models import InvoiceSummary
from ..models import ValidationErrorEntity

from ..operations.invoice import DownloadInvoiceDetails
from ..operations.invoice import (
    DownloadInvoiceDetailsItemTypeEnum,
)
from ..operations.invoice import GenerateInvoiceSummary
from ..operations.invoice import (
    GenerateInvoiceSummaryItemTypeEnum,
)


@same_doc_as(DownloadInvoiceDetails)
def download_invoice_details(
    end_time: str,
    feature: str,
    item_id: str,
    item_type: Union[str, DownloadInvoiceDetailsItemTypeEnum],
    start_time: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Download invoice details (downloadInvoiceDetails)

    Download invoice details as a csv file.
    Other detail info:

      * Returns : invoice details csv file

    Properties:
        url: /platform/admin/namespaces/{namespace}/invoice/details.csv

        method: GET

        tags: ["Invoice"]

        consumes: []

        produces: ["text/csv"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end_time: (endTime) REQUIRED str in query

        feature: (feature) REQUIRED str in query

        item_id: (itemId) REQUIRED str in query

        item_type: (itemType) REQUIRED Union[str, ItemTypeEnum] in query

        start_time: (startTime) REQUIRED str in query

    Responses:
        200: OK - Any (Successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DownloadInvoiceDetails.create(
        end_time=end_time,
        feature=feature,
        item_id=item_id,
        item_type=item_type,
        start_time=start_time,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DownloadInvoiceDetails)
async def download_invoice_details_async(
    end_time: str,
    feature: str,
    item_id: str,
    item_type: Union[str, DownloadInvoiceDetailsItemTypeEnum],
    start_time: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Download invoice details (downloadInvoiceDetails)

    Download invoice details as a csv file.
    Other detail info:

      * Returns : invoice details csv file

    Properties:
        url: /platform/admin/namespaces/{namespace}/invoice/details.csv

        method: GET

        tags: ["Invoice"]

        consumes: []

        produces: ["text/csv"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end_time: (endTime) REQUIRED str in query

        feature: (feature) REQUIRED str in query

        item_id: (itemId) REQUIRED str in query

        item_type: (itemType) REQUIRED Union[str, ItemTypeEnum] in query

        start_time: (startTime) REQUIRED str in query

    Responses:
        200: OK - Any (Successful operation)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DownloadInvoiceDetails.create(
        end_time=end_time,
        feature=feature,
        item_id=item_id,
        item_type=item_type,
        start_time=start_time,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GenerateInvoiceSummary)
def generate_invoice_summary(
    end_time: str,
    feature: str,
    item_id: str,
    item_type: Union[str, GenerateInvoiceSummaryItemTypeEnum],
    start_time: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Generate invoice summary (generateInvoiceSummary)

    Generate invoice summary.
    Other detail info:

      * Returns : query orders

    Properties:
        url: /platform/admin/namespaces/{namespace}/invoice/summary

        method: GET

        tags: ["Invoice"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end_time: (endTime) REQUIRED str in query

        feature: (feature) REQUIRED str in query

        item_id: (itemId) REQUIRED str in query

        item_type: (itemType) REQUIRED Union[str, ItemTypeEnum] in query

        start_time: (startTime) REQUIRED str in query

    Responses:
        200: OK - InvoiceSummary (successful operation)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GenerateInvoiceSummary.create(
        end_time=end_time,
        feature=feature,
        item_id=item_id,
        item_type=item_type,
        start_time=start_time,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GenerateInvoiceSummary)
async def generate_invoice_summary_async(
    end_time: str,
    feature: str,
    item_id: str,
    item_type: Union[str, GenerateInvoiceSummaryItemTypeEnum],
    start_time: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Generate invoice summary (generateInvoiceSummary)

    Generate invoice summary.
    Other detail info:

      * Returns : query orders

    Properties:
        url: /platform/admin/namespaces/{namespace}/invoice/summary

        method: GET

        tags: ["Invoice"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        end_time: (endTime) REQUIRED str in query

        feature: (feature) REQUIRED str in query

        item_id: (itemId) REQUIRED str in query

        item_type: (itemType) REQUIRED Union[str, ItemTypeEnum] in query

        start_time: (startTime) REQUIRED str in query

    Responses:
        200: OK - InvoiceSummary (successful operation)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GenerateInvoiceSummary.create(
        end_time=end_time,
        feature=feature,
        item_id=item_id,
        item_type=item_type,
        start_time=start_time,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# AccelByte Gaming Services Platform Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model
from accelbyte_py_sdk.core import StrEnum

from ..models.discount_category import DiscountCategory
from ..models.discount_item import DiscountItem


class DiscountTypeEnum(StrEnum):
    AMOUNT = "AMOUNT"
    PERCENTAGE = "PERCENTAGE"


class RestrictTypeEnum(StrEnum):
    ITEMS_AND_CATEGORIES = "ITEMS_AND_CATEGORIES"
    NONE = "NONE"


class DiscountConfig(Model):
    """Discount config (DiscountConfig)

    Properties:
        categories: (categories) OPTIONAL List[DiscountCategory]

        currency_code: (currencyCode) OPTIONAL str

        currency_namespace: (currencyNamespace) OPTIONAL str

        discount_amount: (discountAmount) OPTIONAL int

        discount_percentage: (discountPercentage) OPTIONAL int

        discount_type: (discountType) OPTIONAL Union[str, DiscountTypeEnum]

        items: (items) OPTIONAL List[DiscountItem]

        restrict_type: (restrictType) OPTIONAL Union[str, RestrictTypeEnum]

        stackable: (stackable) OPTIONAL bool
    """

    # region fields

    categories: List[DiscountCategory]  # OPTIONAL
    currency_code: str  # OPTIONAL
    currency_namespace: str  # OPTIONAL
    discount_amount: int  # OPTIONAL
    discount_percentage: int  # OPTIONAL
    discount_type: Union[str, DiscountTypeEnum]  # OPTIONAL
    items: List[DiscountItem]  # OPTIONAL
    restrict_type: Union[str, RestrictTypeEnum]  # OPTIONAL
    stackable: bool  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_categories(self, value: List[DiscountCategory]) -> DiscountConfig:
        self.categories = value
        return self

    def with_currency_code(self, value: str) -> DiscountConfig:
        self.currency_code = value
        return self

    def with_currency_namespace(self, value: str) -> DiscountConfig:
        self.currency_namespace = value
        return self

    def with_discount_amount(self, value: int) -> DiscountConfig:
        self.discount_amount = value
        return self

    def with_discount_percentage(self, value: int) -> DiscountConfig:
        self.discount_percentage = value
        return self

    def with_discount_type(self, value: Union[str, DiscountTypeEnum]) -> DiscountConfig:
        self.discount_type = value
        return self

    def with_items(self, value: List[DiscountItem]) -> DiscountConfig:
        self.items = value
        return self

    def with_restrict_type(self, value: Union[str, RestrictTypeEnum]) -> DiscountConfig:
        self.restrict_type = value
        return self

    def with_stackable(self, value: bool) -> DiscountConfig:
        self.stackable = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "categories"):
            result["categories"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.categories
            ]
        elif include_empty:
            result["categories"] = []
        if hasattr(self, "currency_code"):
            result["currencyCode"] = str(self.currency_code)
        elif include_empty:
            result["currencyCode"] = ""
        if hasattr(self, "currency_namespace"):
            result["currencyNamespace"] = str(self.currency_namespace)
        elif include_empty:
            result["currencyNamespace"] = ""
        if hasattr(self, "discount_amount"):
            result["discountAmount"] = int(self.discount_amount)
        elif include_empty:
            result["discountAmount"] = 0
        if hasattr(self, "discount_percentage"):
            result["discountPercentage"] = int(self.discount_percentage)
        elif include_empty:
            result["discountPercentage"] = 0
        if hasattr(self, "discount_type"):
            result["discountType"] = str(self.discount_type)
        elif include_empty:
            result["discountType"] = Union[str, DiscountTypeEnum]()
        if hasattr(self, "items"):
            result["items"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.items
            ]
        elif include_empty:
            result["items"] = []
        if hasattr(self, "restrict_type"):
            result["restrictType"] = str(self.restrict_type)
        elif include_empty:
            result["restrictType"] = Union[str, RestrictTypeEnum]()
        if hasattr(self, "stackable"):
            result["stackable"] = bool(self.stackable)
        elif include_empty:
            result["stackable"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        categories: Optional[List[DiscountCategory]] = None,
        currency_code: Optional[str] = None,
        currency_namespace: Optional[str] = None,
        discount_amount: Optional[int] = None,
        discount_percentage: Optional[int] = None,
        discount_type: Optional[Union[str, DiscountTypeEnum]] = None,
        items: Optional[List[DiscountItem]] = None,
        restrict_type: Optional[Union[str, RestrictTypeEnum]] = None,
        stackable: Optional[bool] = None,
        **kwargs,
    ) -> DiscountConfig:
        instance = cls()
        if categories is not None:
            instance.categories = categories
        if currency_code is not None:
            instance.currency_code = currency_code
        if currency_namespace is not None:
            instance.currency_namespace = currency_namespace
        if discount_amount is not None:
            instance.discount_amount = discount_amount
        if discount_percentage is not None:
            instance.discount_percentage = discount_percentage
        if discount_type is not None:
            instance.discount_type = discount_type
        if items is not None:
            instance.items = items
        if restrict_type is not None:
            instance.restrict_type = restrict_type
        if stackable is not None:
            instance.stackable = stackable
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> DiscountConfig:
        instance = cls()
        if not dict_:
            return instance
        if "categories" in dict_ and dict_["categories"] is not None:
            instance.categories = [
                DiscountCategory.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["categories"]
            ]
        elif include_empty:
            instance.categories = []
        if "currencyCode" in dict_ and dict_["currencyCode"] is not None:
            instance.currency_code = str(dict_["currencyCode"])
        elif include_empty:
            instance.currency_code = ""
        if "currencyNamespace" in dict_ and dict_["currencyNamespace"] is not None:
            instance.currency_namespace = str(dict_["currencyNamespace"])
        elif include_empty:
            instance.currency_namespace = ""
        if "discountAmount" in dict_ and dict_["discountAmount"] is not None:
            instance.discount_amount = int(dict_["discountAmount"])
        elif include_empty:
            instance.discount_amount = 0
        if "discountPercentage" in dict_ and dict_["discountPercentage"] is not None:
            instance.discount_percentage = int(dict_["discountPercentage"])
        elif include_empty:
            instance.discount_percentage = 0
        if "discountType" in dict_ and dict_["discountType"] is not None:
            instance.discount_type = str(dict_["discountType"])
        elif include_empty:
            instance.discount_type = Union[str, DiscountTypeEnum]()
        if "items" in dict_ and dict_["items"] is not None:
            instance.items = [
                DiscountItem.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["items"]
            ]
        elif include_empty:
            instance.items = []
        if "restrictType" in dict_ and dict_["restrictType"] is not None:
            instance.restrict_type = str(dict_["restrictType"])
        elif include_empty:
            instance.restrict_type = Union[str, RestrictTypeEnum]()
        if "stackable" in dict_ and dict_["stackable"] is not None:
            instance.stackable = bool(dict_["stackable"])
        elif include_empty:
            instance.stackable = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, DiscountConfig]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[DiscountConfig]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[DiscountConfig, List[DiscountConfig], Dict[Any, DiscountConfig]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "categories": "categories",
            "currencyCode": "currency_code",
            "currencyNamespace": "currency_namespace",
            "discountAmount": "discount_amount",
            "discountPercentage": "discount_percentage",
            "discountType": "discount_type",
            "items": "items",
            "restrictType": "restrict_type",
            "stackable": "stackable",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "categories": False,
            "currencyCode": False,
            "currencyNamespace": False,
            "discountAmount": False,
            "discountPercentage": False,
            "discountType": False,
            "items": False,
            "restrictType": False,
            "stackable": False,
        }

    @staticmethod
    def get_enum_map() -> Dict[str, List[Any]]:
        return {
            "discountType": ["AMOUNT", "PERCENTAGE"],
            "restrictType": ["ITEMS_AND_CATEGORIES", "NONE"],
        }

    # endregion static methods
